#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          02-07-2018
//      Created for Project :   FLAME
"""

from sqlalchemy import Column, String, and_
from sqlalchemy.dialects.postgresql import JSONB
from clmcservice.models.meta import Base


class ServiceFunctionChain(Base):
    """
    This class defines the service function chain model of the config API, declaring the relations between individual service functions per service function chain.
    """

    __tablename__ = 'sfchain'  # table name in the PostgreSQL database

    sfc = Column(String, nullable=False, primary_key=True)  # service function chain label
    chain = Column(JSONB, nullable=False)  # the service function chain graph represented by a python dictionary (JSON object essentially)

    @property
    def json(self):
        """
        Converts an instance of a ServiceFunctionChain to JSON format.

        :return: a python dictionary object
        """

        fields = {c.name: getattr(self, c.name) for c in self.__table__.columns}

        return fields

    @staticmethod
    def required_columns():
        """
        Returns the required columns for constructing a valid instance.
        :return: a generator object
        """

        return tuple(column.name for column in ServiceFunctionChain.__table__.columns)

    @staticmethod
    def get(sfc):
        """
        Gets the instance matching the sfc argument

        :param sfc: service function chain id

        :return: the first object from the result set that matches the sfc argument (must be only one)
        """

        return ServiceFunctionChain.query().filter(and_(ServiceFunctionChain.sfc == sfc)).first()

    @staticmethod
    def exists(sfc):
        """
        Checks if an instance matching the sfc exists.

        :param sfc: service function chain id

        :return: True if exists, False otherwise
        """

        return ServiceFunctionChain.get(sfc) is not None
